/*
 *  linux/arch/arm/mach-cpe/arch.c
 *  modified by ivan wang 2004/8/18 01:35pm
 */
#include <linux/tty.h>
#include <linux/delay.h>
#include <linux/pm.h>
#include <linux/init.h>

#include <asm/elf.h>
#include <asm/setup.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/arch/cpe/cpe.h>
#include <asm/arch/hardware.h>
#include <asm/sizes.h>
#include <asm/mach/map.h>

extern void genarch_init_irq(void);

/*
 * Physical     Logical
 * 98100000     f9810000    Power Management
 * 98200000     f9820000    UART 1
 * 98300000     f9830000    UART 2
 * 98400000     f9840000    Timer 1/2
 * 98500000	f9850000    Watchdog
 * 98600000	f9860000    RTC
 * 98800000     f9880000    INTC
 * 96700000     f9670000    MAC 
 * 98700000     f9870000    GPIO
 * 98b00000     f98b0000    SSP1
 * 90400000     f9040000    AHB
 * 90100000     f9010000    AHB
 * 80400000     f0400000    Flash
 * 90c00000     90c00000    PCI base/IO
 * b0000000     b0000000    PCI memory resource
 * fb080000     b0800000    A321 Interrupt Controller
 *
 */

static struct map_desc cpe_io_desc[] __initdata = {
 { CPE_PMU_VA_BASE,     CPE_PMU_BASE,       SZ_4K, DOMAIN_IO, 0, 1},
 { CPE_UART1_VA_BASE,   CPE_UART1_BASE,     SZ_4K, DOMAIN_IO, 0, 1}, 
 { CPE_UART2_VA_BASE,   CPE_UART2_BASE,     SZ_4K, DOMAIN_IO, 0, 1},
 { CPE_TIMER1_VA_BASE,  CPE_TIMER1_BASE,    SZ_4K, DOMAIN_IO, 0, 1},
 { CPE_TIMER2_VA_BASE,  CPE_TIMER2_BASE,    SZ_4K, DOMAIN_IO, 0, 1},
 { CPE_TIMER3_VA_BASE,  CPE_TIMER3_BASE,    SZ_4K, DOMAIN_IO, 0, 1},
 { CPE_WDT_VA_BASE,	    CPE_WDT_BASE,	    SZ_4K, DOMAIN_IO, 0, 1},
 { CPE_RTC_VA_BASE,	    CPE_RTC_BASE,	    SZ_4K, DOMAIN_IO, 0, 1},
 { CPE_IC_VA_BASE,      CPE_IC_BASE,        SZ_4K, DOMAIN_IO, 0, 1},
 { CPE_FTMAC_VA_BASE,   CPE_FTMAC_BASE,     SZ_4K, DOMAIN_IO, 0, 1},
 { CPE_FTMAC2_VA_BASE,  CPE_FTMAC_BASE,     SZ_4K, DOMAIN_IO, 0, 1},  //2nd MAC
 { CPE_HOST11_VA_BASE,  CPE_HOST11_BASE,    SZ_4K, DOMAIN_IO, 0, 1},  //USB Host 1.1
 { CPE_GPIO_VA_BASE,    CPE_GPIO_BASE,      SZ_4K, DOMAIN_IO, 0, 1},
 { CPE_SSP1_VA_BASE,    CPE_SSP1_BASE,      SZ_4K, DOMAIN_IO, 0, 1},
 { CPE_SSP2_VA_BASE,    CPE_SSP2_BASE,      SZ_4K, DOMAIN_IO, 0, 1},
 { CPE_A321_SSP_VA_BASE,CPE_A321_SSP_BASE,  SZ_4K, DOMAIN_IO, 0, 1},   // I2S on A321
 { CPE_AHBDMA_VA_BASE,  CPE_AHBDMA_BASE,    SZ_4K, DOMAIN_IO, 0, 1},
 { CPE_AHB_VA_BASE,     CPE_AHB_BASE,       SZ_4K, DOMAIN_IO, 0, 1},
 { CPE_FLASH_VA_BASE,   CPE_FLASH_BASE,     CPE_FLASH_SZ,  DOMAIN_IO, 0, 1}, 
 { CPE_PCI_VA_MEM,      CPE_PCI_MEM,        PCI_MEM_SIZE,  DOMAIN_IO, 0, 1},
 { CPE_PCI_VA_BASE,     CPE_PCI_BASE,       PCI_MEM_SIZE,  DOMAIN_IO, 0, 1},  
 { CPE_A321_IC_VA_BASE, CPE_A321_IC_BASE,   SZ_4K, DOMAIN_IO, 0, 1},
 { CPE_HOST20_VA_BASE,  CPE_HOST20_BASE,    SZ_4K, DOMAIN_IO, 0, 1},
 { CPE_USBDEV_VA_BASE,  CPE_USBDEV_BASE,    SZ_4K, DOMAIN_IO, 0, 1},   //USB device
 { CPE_FOTG200_VA_BASE,  CPE_FOTG200_BASE,    SZ_4K, DOMAIN_IO, 0, 1}, //FOTG200

 { CPE_CF_VA_BASE,      CPE_CF_BASE,        SZ_4K, DOMAIN_IO, 0, 1},   //CF
 { CPE_SMMC_VA_BASE,    CPE_SMMC_BASE,      SZ_4K, DOMAIN_IO, 0, 1},   //SMMC
 { CPE_MMSC_VA_BASE,    CPE_MMSC_BASE,      SZ_4K, DOMAIN_IO, 0, 1},   //Memory Stick
 { CPE_IDE_VA_BASE,     CPE_IDE_BASE,       SZ_4K, DOMAIN_IO, 0, 1},   //IDE    
 { CPE_SD_VA_BASE,      CPE_SD_BASE,        SZ_4K, DOMAIN_IO, 0, 1},   //SD
 { CPE_SD_VA_BASE,      CPE_SD_BASE,        SZ_4K, DOMAIN_IO, 0, 1},   //SD

 { CPE_APBDMA_VA_BASE,	CPE_APBDMA_BASE,	SZ_4K, DOMAIN_IO, 0, 1},   //APB_DMA
 { CPE_SECU_VA_BASE,	CPE_SECU_BASE,	    SZ_4K, DOMAIN_IO, 0, 1},   //security AES/DES/3DES
 { CPE_KBD_VA_BASE,     CPE_KBD_BASE,       SZ_4K, DOMAIN_IO, 0, 1},   //Keyboard
 { CPE_MOUSE_VA_BASE,   CPE_MOUSE_BASE,     SZ_4K, DOMAIN_IO, 0, 1},   //Mouse
 { CPE_LCD_VA_BASE,     CPE_LCD_BASE,       SZ_4K, DOMAIN_IO, 0, 1},   //LCD
 { CPE_80PORT_VA_BASE,  CPE_80PORT_BASE,    SZ_4K, DOMAIN_IO, 0, 1},   //80 port 
 LAST_DESC
};

void __init cpe_map_io(void)
{
	iotable_init(cpe_io_desc);
}

static void __init fixup_cpe(struct machine_desc *desc, struct param_struct *unused,
                             char **cmdline, struct meminfo *mi)
{
    mi->nr_banks      = 1;
    mi->bank[0].start = DRAM_BASE;
    mi->bank[0].size  = DRAM_SIZE;
    mi->bank[0].node  = 0;
}

MACHINE_START(CPE, "Faraday CPE")
       MAINTAINER("Faraday")
       BOOT_MEM(0x00000000, 0x982000000, 0xf9820000) //non-workable varible but booting procedure will check it
       BOOT_PARAMS(0x00000100)
       FIXUP(fixup_cpe)
       MAPIO(cpe_map_io)
       INITIRQ(genarch_init_irq)
MACHINE_END
