#include <linux/module.h>
#include <linux/init.h>

#include <asm/mach/irq.h>
#include <asm/hardware.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/system.h>
#include <asm/arch/cpe/cpe.h>
#include <asm/arch/ftpci.h>

static spinlock_t cpe_int_lock;

inline void cpe_irq_set_mode(unsigned int base_p,unsigned int irq,unsigned int edge)
{	
    if(edge)
    	*(volatile unsigned int *)(base_p+IRQ_MODE_REG)|=(1<<irq);
    else
        *(volatile unsigned int *)(base_p+IRQ_MODE_REG)&=~(1<<irq);	
}	

inline void cpe_irq_set_level(unsigned int base_p,unsigned int irq,unsigned int low)
{	
    if(low) 
        *(volatile unsigned int *)(base_p+IRQ_LEVEL_REG)|=(1<<irq);
    else
    	*(volatile unsigned int *)(base_p+IRQ_LEVEL_REG)&=~(1<<irq);
}	


inline void cpe_fiq_set_mode(unsigned int base_p,unsigned int fiq,unsigned int edge)
{
    if(edge) 
    	*(volatile unsigned int *)(base_p+FIQ_MODE_REG)|=(1<<fiq);
    else
        *(volatile unsigned int *)(base_p+FIQ_MODE_REG)&=~(1<<fiq);
}	


inline void cpe_fiq_set_level(unsigned int base_p,unsigned int fiq,unsigned int low)
{
    if(low) 
        *(volatile unsigned int *)(base_p+FIQ_LEVEL_REG)|=(1<<fiq);
    else
    	*(volatile unsigned int *)(base_p+FIQ_LEVEL_REG)&=~(1<<fiq);
}	


void cpe_int_set_irq(unsigned int irq,int mode,int level)
{
    unsigned long   flags;
    
    //printk("cpe_int_set_irq:%d\n",irq);
    
    spin_lock_irqsave(&cpe_int_lock, flags);
    
   	if(irq<32) //irq
   	{
		cpe_irq_set_mode(CPE_IC_VA_BASE,irq ,mode);
		cpe_irq_set_level(CPE_IC_VA_BASE,irq,level);
		goto cpe_int_set_irq_exit;
   	}		
    if(irq<64) //fiq
    {
	   	irq-=32;
		cpe_fiq_set_mode(CPE_IC_VA_BASE,irq,mode);
		cpe_fiq_set_level(CPE_IC_VA_BASE,irq,level);
		goto cpe_int_set_irq_exit;
	}
#ifdef CONFIG_A321_BOARD
    if(irq<96) //a321 irq
    {
	   	irq-=64;
		cpe_irq_set_mode(CPE_A321_IC_VA_BASE,irq,mode);
		cpe_irq_set_level(CPE_A321_IC_VA_BASE,irq,level);
		cpe_irq_set_mode(CPE_IC_VA_BASE,IRQ_EXT_A321,LEVEL);
		cpe_irq_set_level(CPE_IC_VA_BASE,IRQ_EXT_A321,H_ACTIVE);
		goto cpe_int_set_irq_exit;
	}
    if(irq<150) //a321 fiq
    {
	   	irq-=96;
		cpe_fiq_set_mode(CPE_A321_IC_VA_BASE,irq,mode);
		cpe_fiq_set_level(CPE_A321_IC_VA_BASE,irq,level);
		cpe_fiq_set_mode(CPE_IC_VA_BASE,IRQ_EXT_A321,LEVEL);
		cpe_fiq_set_level(CPE_IC_VA_BASE,IRQ_EXT_A321,H_ACTIVE);
		goto cpe_int_set_irq_exit;
	}
#endif

#ifdef CONFIG_PCI
    //pci virtual irq
    if((irq>=CPE_VIRQ_PCI_START) && (irq<=CPE_VIRQ_PCI_END))
    {
        if(!ftpci_probed)
            goto cpe_int_set_irq_exit;
#ifdef CONFIG_A321_BOARD
		cpe_irq_set_mode(CPE_A321_IC_VA_BASE,IRQ_A321_PCI,mode);
		cpe_irq_set_level(CPE_A321_IC_VA_BASE,IRQ_A321_PCI,level);
		cpe_irq_set_mode(CPE_IC_VA_BASE,IRQ_EXT_A321,LEVEL);
		cpe_irq_set_level(CPE_IC_VA_BASE,IRQ_EXT_A321,H_ACTIVE);
#else
		cpe_irq_set_mode(CPE_IC_VA_BASE,IRQ_PCI,mode);
		cpe_irq_set_level(CPE_IC_VA_BASE,IRQ_PCI,level);
#endif   
        goto cpe_int_set_irq_exit;
    }
#endif

    if((irq>=CPE_VIRQ_AHBDMA_START) && (irq<=CPE_VIRQ_AHBDMA_END))
    {
        cpe_irq_set_mode(CPE_IC_VA_BASE,IRQ_CPE_AHB_DMA,mode);
		cpe_irq_set_level(CPE_IC_VA_BASE,IRQ_CPE_AHB_DMA,level);
		goto cpe_int_set_irq_exit;
    }
    
    if((irq>=CPE_VIRQ_APBDMA_START) && (irq<=CPE_VIRQ_APBDMA_END))
    {
        cpe_irq_set_mode(CPE_IC_VA_BASE,IRQ_CPE_APB_DMA,mode);
		cpe_irq_set_level(CPE_IC_VA_BASE,IRQ_CPE_APB_DMA,level);
		goto cpe_int_set_irq_exit;
    }

    printk("Not support irq %d\n",irq);

cpe_int_set_irq_exit:
    spin_unlock_irqrestore(&ftpci_lock, flags);    
}

void cpe_int_clear_irq(unsigned int base,unsigned int irq)
{
    *(volatile unsigned int *)(base+IRQ_CLEAR_REG)=1<<irq;
}

void cpe_int_clear_fiq(unsigned int base,unsigned int irq)
{	
    *(volatile unsigned int *)(base+FIQ_CLEAR_REG)=1<<irq;
}

inline void cpe_int_disable_irq(unsigned int base,unsigned int irq)
{
    *(volatile unsigned int *)(base+IRQ_MASK_REG)&=~(1<<irq);
}

inline void cpe_int_disable_fiq(unsigned int base,unsigned int irq)
{
    *(volatile unsigned int *)(base+FIQ_MASK_REG)&=~(1<<irq);
}


/*  Turn the interrupt source on. */
inline void cpe_int_enable_irq(unsigned int base,unsigned int irq)
{
    *(volatile unsigned int *)(base+IRQ_MASK_REG)|=(1<<irq);
}

inline void cpe_int_enable_fiq(unsigned int base,unsigned int irq)
{    
	*(volatile unsigned int *)(base+FIQ_MASK_REG)|=(1<<irq);
}

void cpe_unmask_irq(unsigned int irq)
{
    unsigned long   flags;
#if 0 //debug IRQ
if((irq!=IRQ_TIMER1)&&(irq!=IRQ_UART1))
    printk("[%d]",irq);
#endif
    spin_lock_irqsave(&cpe_int_lock, flags);
    
	if(irq<32) //irq
	{
		cpe_int_clear_irq(CPE_IC_VA_BASE,irq);
		cpe_int_enable_irq(CPE_IC_VA_BASE,irq);
		goto cpe_unmask_irq_exit;
	}
	if(irq<64) //fiq
	{
	    irq-=32;
		cpe_int_clear_fiq(CPE_IC_VA_BASE,irq);
		cpe_int_enable_fiq(CPE_IC_VA_BASE,irq);
		goto cpe_unmask_irq_exit;
	}
#ifdef CONFIG_A321_BOARD	
    if(irq<96) //a321 irq
    {
	   	irq-=64;
		cpe_int_clear_irq(CPE_A321_IC_VA_BASE,irq);
		cpe_int_clear_irq(CPE_IC_VA_BASE,IRQ_EXT_A321);
		cpe_int_enable_irq(CPE_A321_IC_VA_BASE,irq);
		cpe_int_enable_irq(CPE_IC_VA_BASE,IRQ_EXT_A321);
		goto cpe_unmask_irq_exit;
    }
    if(irq<150) //a321 fiq
    {
	   	irq-=96;
		cpe_int_clear_fiq(CPE_A321_IC_VA_BASE,irq);
		cpe_int_clear_fiq(CPE_IC_VA_BASE,IRQ_EXT_A321);
        cpe_int_enable_fiq(CPE_A321_IC_VA_BASE,irq);
		cpe_int_enable_fiq(CPE_IC_VA_BASE,IRQ_EXT_A321);
		goto cpe_unmask_irq_exit;
    }
#endif
    
#ifdef CONFIG_PCI
    //pci virtual irq
    if((irq>=CPE_VIRQ_PCI_START) && (irq<=CPE_VIRQ_PCI_END))
    {
	    //printk("enable irq:%d probed:%d\n",IRQ_PCI,ftpci_probed);
        if(!ftpci_probed)
            goto cpe_unmask_irq_exit;
        ftpci_clear_irq(irq-150);
#ifdef CONFIG_A321_BOARD
        cpe_int_clear_irq(CPE_A321_IC_VA_BASE,IRQ_A321_PCI);
        cpe_int_enable_irq(CPE_A321_IC_VA_BASE,IRQ_A321_PCI); //always enabled
		cpe_int_clear_irq(CPE_IC_VA_BASE,IRQ_EXT_A321);
		//cpe_int_enable_irq(CPE_IC_VA_BASE,IRQ_EXT_A321); //always enabled
#else
		cpe_int_clear_irq(CPE_IC_VA_BASE,IRQ_PCI);
		cpe_int_enable_irq(CPE_IC_VA_BASE,IRQ_PCI); //always enabled
#endif
		goto cpe_unmask_irq_exit;
    }    
#endif

    if((irq>=CPE_VIRQ_AHBDMA_START) && (irq<=CPE_VIRQ_AHBDMA_END))
    {
        cpe_int_enable_irq(CPE_IC_VA_BASE,IRQ_CPE_AHB_DMA);
		goto cpe_unmask_irq_exit;
    }

    if((irq>=CPE_VIRQ_APBDMA_START) && (irq<=CPE_VIRQ_APBDMA_END))
    {
        cpe_int_enable_irq(CPE_IC_VA_BASE,IRQ_CPE_APB_DMA);
		goto cpe_unmask_irq_exit;
    }

cpe_unmask_irq_exit:
    spin_unlock_irqrestore(&ftpci_lock, flags);    
}



void cpe_mask_ack_irq(unsigned int irq)
{
    unsigned long   flags;
       
    spin_lock_irqsave(&cpe_int_lock, flags);
#if 0 //debug IRQ
if((irq!=IRQ_TIMER1)&&(irq!=IRQ_UART1))
    printk("<%d>",irq);
#endif
	if(irq<32) //irq
	{    
        cpe_int_disable_irq(CPE_IC_VA_BASE,irq);
        goto cpe_mask_ack_irq_exit;
    }
	if(irq<64) //fiq
	{
	    irq-=32;
	    cpe_int_disable_fiq(CPE_IC_VA_BASE,irq);
	    goto cpe_mask_ack_irq_exit;
	}
#ifdef CONFIG_A321_BOARD	
    if(irq<96) //a321 irq
    {
	   	irq-=64;
	   	cpe_int_disable_irq(CPE_A321_IC_VA_BASE,irq);
	   	goto cpe_mask_ack_irq_exit;
	}
    if(irq<150) //a321 fiq
    {
	   	irq-=96;
	   	cpe_int_disable_fiq(CPE_A321_IC_VA_BASE,irq);
	   	goto cpe_mask_ack_irq_exit;
    }
#endif

#ifdef CONFIG_PCI
    //pci virtual irq
    if((irq>=CPE_VIRQ_PCI_START) && (irq<=CPE_VIRQ_PCI_END))
    {
	    //printk("disable irq:%d probed:%d\n",IRQ_PCI,ftpci_probed);
        if(!ftpci_probed)
            goto cpe_mask_ack_irq_exit;
#ifdef CONFIG_A321_BOARD
        cpe_int_disable_irq(CPE_A321_IC_VA_BASE,IRQ_A321_PCI);
#else
        cpe_int_disable_irq(CPE_IC_VA_BASE,IRQ_PCI);
#endif
    }    
#endif

    if((irq>=CPE_VIRQ_AHBDMA_START) && (irq<=CPE_VIRQ_AHBDMA_END))
    {
        cpe_int_disable_irq(CPE_IC_VA_BASE,IRQ_CPE_AHB_DMA);
		goto cpe_mask_ack_irq_exit;
    }

    if((irq>=CPE_VIRQ_APBDMA_START) && (irq<=CPE_VIRQ_APBDMA_END))
    {
        cpe_int_disable_irq(CPE_IC_VA_BASE,IRQ_CPE_APB_DMA);
		goto cpe_mask_ack_irq_exit;
    }

cpe_mask_ack_irq_exit:
    spin_unlock_irqrestore(&ftpci_lock, flags);    
}

void cpe_mask_irq(unsigned int irq)
{
    cpe_mask_ack_irq(irq);
}

void cpe_int_init(void)
{
    spin_lock_init(&cpe_int_lock);
    //init interrupt controller
	outl(0, CPE_IC_VA_BASE+IRQ_MASK_REG);
	outl(0, CPE_IC_VA_BASE+FIQ_MASK_REG);
	outl(0xffffffff, CPE_IC_VA_BASE+IRQ_CLEAR_REG);
	outl(0xffffffff, CPE_IC_VA_BASE+FIQ_CLEAR_REG);
#ifdef CONFIG_A321_BOARD
    printk("Faraday A321 Interrupt Controller Enabled\n");
    //init a321 interrupt controller
	outl(0, CPE_A321_IC_VA_BASE+IRQ_MASK_REG);
	outl(0, CPE_A321_IC_VA_BASE+FIQ_MASK_REG);
	outl(0xffffffff, CPE_A321_IC_VA_BASE+IRQ_CLEAR_REG);
	outl(0xffffffff, CPE_A321_IC_VA_BASE+FIQ_CLEAR_REG);
    cpe_int_set_irq(IRQ_EXT_A321, LEVEL, H_ACTIVE);
    cpe_int_enable_irq(CPE_IC_VA_BASE,IRQ_EXT_A321);
#endif
}

EXPORT_SYMBOL(cpe_int_set_irq);
EXPORT_SYMBOL(cpe_int_clear_irq);

