
/* SPI related command functions */

#include <linux/module.h>
#include <asm/io.h>
#include <asm/hardware.h>
#include <linux/mtd/nor.h> 


int spi_init(void)
{
	return 0;
}

int spi_reset(void)
{
	return 0;
}

int spi_read_id(u8 *id)
{
	writel(0x9F,DATA_CFG1);
	*id=readw(PIO_DATA_REG);

	return 0;
}

int spi_cmd_erase(unsigned long addr, int mode)
{
	int rc = 0;
	unsigned char stat;

	switch ( mode ) {
	case DMA_START:
		writel(0x06,DATA_CFG1); //WREN
		rc = pl_nor_polling_intr(NOR_CMD_INTR_MASK, 0);
		if ( rc == 0 ){
			rc = -1;
			break;
		}
		writel(addr,NOR_ADDR_REG);
		writel(0xD8,DATA_CFG1);
	
		break;
	case DMA_END:
		/* nothing to do */
		break;
	}
	
	return rc;
}

int spi_cmd_write(unsigned long addr, int size, int mode)
{
	int rc = 0;
	
	switch ( mode ) {
	case DMA_START:
		writel(0x06,DATA_CFG1); //WREN
		rc = pl_nor_polling_intr(NOR_CMD_INTR_MASK, 0);
		if ( rc == 0 ){
			rc = -1;
			break;
		}
		writel(addr, NOR_ADDR_REG);//start address	
		writel(0x80000000|size, DMA_S_REG); 
		break;
	case DMA_END:
		/* nothing to do */
		break;
	}
	                                         
	return rc;
}

int spi_cmd_read(unsigned long addr, int size, int mode)
{
	int rc = 0;
	
	switch ( mode ) {
	case DMA_START:
		writel(addr, NOR_ADDR_REG);
		writel(0x40000000|size, DMA_S_REG);
		break;
	case DMA_END:
		/* nothing to do */
		break;
	}
	
	return rc;
}

int spi_pio_read(unsigned long addr, char *buf, int size)
{
	char *p;
	int val, remains, count, rc;
	
	rc = 0;
	remains = size;
	p = buf;

	while ( remains > 0 ) {
		writel(addr, NOR_ADDR_REG);
		writel(0x03, DATA_CFG1);		
		rc = pl_nor_polling_intr(NOR_CMD_INTR_MASK, 0);
		if ( rc == 0 ){
			rc = -1;
			break;
		}
		if ( remains >= 4 ) {
			writel(readl(PIO_DATA_REG), p);
			count = 4;
		} else {
			val = readl(PIO_DATA_REG);
			memcpy(p, (char *)&val, remains);
			count = remains;
		}
		addr += count;
		p += count;
		remains -= count;
	}
	
	return rc;
}

int spi_pio_write(unsigned long addr, char *buf, int size)
{
	char *p;
	int val, remains, count, rc;

	rc = 0;
	remains = size;
	p = buf;
	
	while ( remains > 0 ) {
		writel(0x06,DATA_CFG1); //WREN
		rc = pl_nor_polling_intr(NOR_CMD_INTR_MASK, 0);
		if ( rc == 0 ){
			rc = -1;
			break;
		}
		writel(addr, NOR_ADDR_REG);
		if ( remains >= 4 ) {
			writel(readl(p), PIO_DATA_REG);
			count = 4;
		} else {
			memset((char *)&val, 0xff, 4);
			memcpy((char *)&val, p, remains);
			count = remains;
		}
		rc = pl_nor_polling_intr(NOR_CMD_INTR_MASK, 0);
		if ( rc == 0 ){
			rc = -1;
			break;
		}
		addr += count;
		p += count;
		remains -= count;
	}
	
	return rc;
}


EXPORT_SYMBOL(spi_init);
EXPORT_SYMBOL(spi_reset);
EXPORT_SYMBOL(spi_read_id);
EXPORT_SYMBOL(spi_cmd_erase);
EXPORT_SYMBOL(spi_cmd_write);
EXPORT_SYMBOL(spi_cmd_read);
EXPORT_SYMBOL(spi_pio_read);
EXPORT_SYMBOL(spi_pio_write);
